#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#ifdef SW
#include <Pipes.h>
#include <pipeHandler.h>
#include "prog.h"
#else
#include "vhdlCStubs.h"
#endif

void Exit(int sig)
{
	fprintf(stderr, "## Break! ##\n");
	exit(0);
}

void *accumulate_(void* fargs)
{
   accumulate();
}

void *write_pipe_(void* a)
{
   write_uint32_n("in_data",(uint32_t*)a, 10);
}

void *read_pipe_(void* a)
{
   read_uint32_n("out_data",(uint32_t*)a, 10);
}


int main(int argc, char* argv[])
{
	signal(SIGINT,  Exit);
  	signal(SIGTERM, Exit);

	uint32_t data_in[10], data_out[10];
        int i;

#ifdef SW

	// in the "software" case, we start a pipe-handler
	// to manage the pipes..  Earlier, we were using named
	// pipes, which were very flaky and difficult to debug.
	// the pipeHandler generates a log file (pipeHandler.log)
	// which is very useful for figuring out what happened.
	// one can also use gdb to trace activity.
	init_pipe_handler();

	usleep(100);
	
	// register two FIFOs..
	register_pipe("in_data",10,32,0);
	register_pipe("out_data",10,32,0);

#endif
	
#ifndef SW
	// to set the initial value of sum.
	// in the hardware version, storage
	// variables are initialized by calling
	// this function (auto-generated by
	// the Aa linker AaLinkExtMem)
	global_storage_initializer_();
#endif



        for(i = 0; i < 10; i++)
	{
		data_in[i] = i;
	}

	pthread_t acc_t, wpipe_t, rpipe_t;

#ifdef SW

	pthread_create(&acc_t,NULL,&accumulate_,NULL);
#endif
	pthread_create(&wpipe_t,NULL,&write_pipe_,(void*)data_in);
	pthread_create(&rpipe_t,NULL,&read_pipe_,(void*)data_out);


	pthread_join(wpipe_t,NULL);
	pthread_join(rpipe_t,NULL);


 	fprintf(stdout,"from out_data, we read ");
	for(i=0; i < 10; i++)
 	  fprintf(stdout," %u ", data_out[i]);
	fprintf(stdout,"\n");

#ifdef SW
	pthread_cancel(acc_t);
	close_pipe_handler();
#endif
}
