//
// Copyright (C) 2010-: Madhav P. Desai
// All Rights Reserved.
//  
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal with the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimers.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimers in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the names of the AHIR Team, the Indian Institute of
//    Technology Bombay, nor the names of its contributors may be used
//    to endorse or promote products derived from this Software
//    without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
// ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <pthreadUtils.h>

#ifdef SW
#include <Pipes.h>
#include <pipeHandler.h>
#include "prog.h"
#else
#include "vhdlCStubs.h"
#endif

void Exit(int sig)
{
	fprintf(stderr, "## Break! ##\n");
	exit(0);
}

#ifdef SW
DEFINE_THREAD(accumulate)
#endif

int main(int argc, char* argv[])
{
	signal(SIGINT,  Exit);
  	signal(SIGTERM, Exit);

	uint32_t data_in[10], data_out[10];
        int i;

#ifdef SW
	PTHREAD_DECL(accumulate)
	PTHREAD_CREATE(accumulate)

	init_pipe_handler();

	// register FIFO
	register_pipe("in_data",10,32,0);

	// register FIFO..
	register_pipe("out_data",10,32,0);

#endif
	
#ifndef SW
	// to set the initial value of sum.
	// in the hardware version, storage
	// variables are initialized by calling
	// this function (auto-generated by
	// the Aa linker AaLinkExtMem)
	global_storage_initializer_();
#endif



        for(i = 0; i < 10; i++)
	{
		data_in[i] = i;
	}

	// write 10 things to in_data, it has enough room..
        write_uint32_n("in_data",(uint32_t*)data_in, 10);

	// read back 10 things from out_data..
	read_uint32_n("out_data",(uint32_t*)data_out, 10);


 	fprintf(stdout,"from out_data, we read ");
	for(i=0; i < 10; i++)
 	  fprintf(stdout," %u ", data_out[i]);
	fprintf(stdout,"\n");

      
	fprintf(stdout,"Sum 0 is %d\n",get_sum(0));
	fprintf(stdout,"Sum 1 is %d\n",get_sum(1));

#ifdef SW
	close_pipe_handler();
#endif
}
